@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2012-2013  Tetsuo Handa
  *
- * Version: 0.1   2013/01/06
+ * Version: 0.2   2016/10/05
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/mount.h>
 #include <sys/ptrace.h>
 #include <sys/socket.h>
@@ -364,12 +365,12 @@ static void startup(void)
 {
 	int pipe_fd[2] = { EOF, EOF };
 	static char buffer[1048576];
-	FILE *fp = fopen("/proc/caitsith/policy", "r");
-	policy_fd = open("/proc/caitsith/policy", O_WRONLY);
-	audit_fd = open("/proc/caitsith/audit", O_RDONLY);
+	FILE *fp = fopen(POLDIR "/policy", "r");
+	policy_fd = open(POLDIR "/policy", O_WRONLY);
+	audit_fd = open(POLDIR "/audit", O_RDONLY);
 	if (!fp || policy_fd == EOF || audit_fd == EOF) {
 		fprintf(stderr,
-			"***** Can't open /proc/caitsith/ interface.\n");
+			"***** Can't open " POLDIR "/ interface.\n");
 		exit(1);
 	}
 	if (pipe(pipe_fd)) {
@@ -405,7 +406,7 @@ static void startup(void)
 		close(policy_fd);
 		close(audit_fd);
 		close(pipe_fd[1]);
-		fd = open("/proc/caitsith/query", O_RDWR);
+		fd = open(POLDIR "/query", O_RDWR);
 		while (1) {
 			unsigned int serial;
 			unsigned int retry;
@@ -664,7 +665,12 @@ static int test_unmount(void)
 
 static int test_pivot_root(void)
 {
-	pivot_root("/proc/", "/proc/caitsith/");
+	errno = 0;
+	pivot_root("/sys/kernel/security/", "/sys/kernel/security/caitsith/");
+	if (errno == ENOENT) {
+		errno = 0;
+		pivot_root("/proc/", "/proc/caitsith/");
+	}
 	return errno;
 }
 
@@ -1210,6 +1216,32 @@ static int test_use_new_kernel(void)
 	return errno;
 }
 
+static void reset_policy(void)
+{
+	FILE *fp2 = fopen(POLDIR "/policy", "r");
+	FILE *fp1 = fopen(POLDIR "/policy", "w");
+	if (!fp1 || !fp2) {
+		fprintf(stderr, " Can't open " POLDIR "/policy\n");
+		exit(1);
+	}
+	while (1) {
+		const int c = fgetc(fp2);
+		if (c == EOF)
+			break;
+		fputc(c, fp1);
+		if (c == '\n')
+			fprintf(fp1, "delete ");
+	}
+	fclose(fp2);
+	fclose(fp1);
+
+	/* Do not leave the init process in stopped state. */
+	kill(1, SIGCONT);
+	
+	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
+	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
+}
+
 int main(int argc, char *argv[])
 {
 	int i;
@@ -1297,8 +1329,7 @@ int main(int argc, char *argv[])
 		{ NULL, NULL }
 	};
 
-	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
-	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
+	reset_policy();
 
 	startup();
 	for (i = 0; testcases[i].action; i++)
